# encoding: utf8
"""
query.py
@author Meng.yangyang
@description 信息查询
@created Mon Jan 07 2019 16:50:59 GMT+0800 (CST)
@last-modified Thu Jan 10 2019 09:35:56 GMT+0800 (CST)
"""

import re
import copy
import json
import logging

from hack12306.constants import SEAT_TYPE_CODE_MAP
from hack12306.query import TrainInfoQueryAPI
from hack12306.constants import SEAT_TYPE_CODE_MAP

from . import exceptions

_logger = logging.getLogger('booking')


def query_left_tickets(train_date, from_station, to_station, seat_types, train_name=None):
    """
    信息查询-剩余车票
    :param train_date
    :param from_station
    :param to_station
    :param seat_types
    :param train_name
    :return JSON 对象
    """
    date_pattern = re.compile(r'^\d{4}-\d{2}-\d{2}$')
    assert date_pattern.match(train_date), 'Invalid train_date param. %s' % train_date

    assert isinstance(seat_types, list), u'Invalid seat_types param. %s' % seat_types
    assert frozenset(seat_types) <= frozenset(dict(SEAT_TYPE_CODE_MAP).keys()
                                              ), u'Invalid seat_types param. %s' % seat_types

    train_info = {}

    trains = TrainInfoQueryAPI().info_query_left_tickets(train_date, from_station, to_station)
    # _logger.debug('query left tickets. %s' % json.dumps(trains, ensure_ascii=False))
    if train_name:
        for train in trains:
            if train['train_name'] == train_name:
                train_info = copy.deepcopy(train)
                break
        else:
            raise exceptions.BookingTrainNoLeftTicket()
    else:
        # TODO Optimize multiple trains select type
        for train in trains:
            for seat_type in seat_types:
                if train.get(seat_type, ''):
                    train_info = copy.deepcopy(train)
                    break
        else:
            raise exceptions.BookingTrainNoLeftTicket()

    if not  train_info:
        raise exceptions.BookingTrainNoLeftTicket()

    _logger.debug('query left tickets train info. %s' % json.dumps(train_info, ensure_ascii=False))

    select_seat_type = None
    for seat_type in seat_types:
        seat_type_left_ticket = train_info.get(seat_type, '')
        if seat_type_left_ticket and seat_type_left_ticket != u'无':
            select_seat_type = seat_type
            break
    else:
        raise exceptions.BookingTrainNoLeftTicket()

    result = {
        'train_date': train_date,
        'from_station': train_info['from_station'],
        'to_station': train_info['to_station'],
        'seat_type': select_seat_type,
        'seat_type_code': dict(SEAT_TYPE_CODE_MAP)[select_seat_type],
        'departure_time': train_info['departure_time'],
        'arrival_time': train_info['arrival_time'],
        'secret': train_info['secret'],
        'train_name': train_info['train_name'],
        'duration': train_info['duration'],
        'train_num': train_info['train_num']
    }
    return result


def query_station_code_map():
    """
    信息查询-查询车站编码列表
    :return JSON对象
    """
    station_code_map = {}

    stations = TrainInfoQueryAPI().info_query_station_list()
    for station in stations:
        station_code_map[station['name']] = station['code']

    return station_code_map



# encoding: utf8
"""
auth.py
@author Meng.yangyang
@description 认证
@created Mon Jan 07 2019 16:35:01 GMT+0800 (CST)
@last-modified Wed Jan 09 2019 16:26:20 GMT+0800 (CST)
"""


import os
import time
import uuid
import json
import base64
import logging
from PIL import Image

from hack12306.auth import TrainAuthAPI
from hack12306.user import TrainUserAPI
from hack12306.exceptions import TrainUserNotLogin

from . import settings

_logger = logging.getLogger('booking')


def auth_is_login(cookies=None):
    """
    检查用户是否登录
    :param cookies JSON对象
    :return True已登录, False未登录
    """
    return TrainAuthAPI().auth_check_login(cookies=cookies)


def auth_qr():
    """
    认证-二维码登录
    """
    try:
        train_auth_api = TrainAuthAPI()

        _logger.debug('1. auth init')
        cookie_dict = train_auth_api.auth_init()

        _logger.debug('2. auth get qr')
        result = train_auth_api.auth_qr_get(cookies=cookie_dict)
        assert isinstance(result, dict)
        qr_uuid = result['uuid']
        qr_img_path = '/tmp/12306/booking/login-qr-%s.jpeg' % uuid.uuid1().hex

        if not os.path.exists(os.path.dirname(qr_img_path)):
            os.makedirs(os.path.dirname(qr_img_path))

        with open(qr_img_path, 'wb') as f:
            f.write(base64.b64decode(result['image']))

        # open qr image with browser
        cmd = settings.CHROME_APP_OPEN_CMD.format(filepath=qr_img_path)
        os.system(cmd)

        _logger.debug('3. auth check qr')
        for _ in range(6):
            _logger.info('请扫描二维码登录！')
            qr_check_result = train_auth_api.auth_qr_check(qr_uuid, cookies=cookie_dict)
            _logger.debug('check qr result. %s' % json.dumps(qr_check_result, ensure_ascii=False))
            if qr_check_result['result_code'] == "2":
                _logger.debug('qr check success result. %s' % json.dumps(qr_check_result, ensure_ascii=False))
                _logger.info('二维码扫描成功！')
                break

            time.sleep(3)
        else:
            _logger.error('二维码扫描失败，重新生成二维码')
            raise TrainUserNotLogin('扫描述二维码失败')

        uamtk_result = train_auth_api.auth_uamtk(qr_check_result['uamtk'], cookies=cookie_dict)
        _logger.debug('4. auth uamtk result. %s' % json.dumps(uamtk_result, ensure_ascii=False))

        uamauth_result = train_auth_api.auth_uamauth(uamtk_result['newapptk'], cookies=cookie_dict)
        _logger.debug('5. auth uamauth result. %s' % json.dumps(uamauth_result, ensure_ascii=False))

        cookies = {
            'tk': uamauth_result['apptk']
        }
        cookies.update(**cookie_dict)
        user_info_result = TrainUserAPI().user_info(cookies=cookies)
        _logger.debug('%s login successfully.' % user_info_result['name'])
        _logger.debug('cookies. %s' % json.dumps(cookies, ensure_ascii=False,))
        _logger.info('%s 登录成功。' % user_info_result['name'].encode('utf8'))

        return cookies
    finally:
        if os.path.exists(qr_img_path):
            os.remove(qr_img_path)



# encoding: utf8
"""
utils.py
@author Meng.yangyang
@description 工具函数
@created Mon Jan 07 2019 13:22:25 GMT+0800 (CST)
@last-modified Wed Jan 09 2019 16:31:28 GMT+0800 (CST)
"""

import os
import json
import platform
import requests
from PIL import Image


def qr_terminal_draw(filepath):
    assert isinstance(filepath, (str))

    if not os.path.exists(filepath):
        raise Exception('file not exists. %s' % filepath)

    if platform.system() == "Windows":
        white_block = '▇'
        black_block = '  '
        new_line = '\n'
    else:
        white_block = '\033[1;37;47m  '
        black_block = '\033[1;37;40m  '
        new_line = '\033[0m\n'

    output = ''
    im = Image.open(filepath)
    im = im.resize((21, 21))

    pixels = im.load()

    output += white_block * (im.width + 2) + new_line
    for h in range(im.height):
        output += white_block
        for w in range(im.width):
            pixel = pixels[w,h]     # NOQA
            if pixel[0] == 0:
                output += black_block
            elif pixel[0] == 255:
                output += white_block
            else:
                assert 'Unsupported pixel. %s' % pixel

        else:
            output += white_block + new_line
    output += white_block * (im.width + 2) + new_line

    return output


def get_public_ip():
    resp = requests.get('http://httpbin.org/ip')
    if resp.status_code != 200:
        raise Exception('Network error')
    return json.loads(resp.content)['origin'].encode('utf8')


def check_seat_types(seat_types):
    from hack12306.constants import SEAT_TYPE_CODE_MAP

    assert isinstance(seat_types, (list, tuple))
    if not frozenset(seat_types) <= frozenset(dict(SEAT_TYPE_CODE_MAP).keys()):
        return False
    return True



# encoding: utf8

import os

INIT_DONE = False

COOKIES = {}
PAY_FILEPATH = './{date}-{order_no}-{bank_id}.html'
STATION_CODE_MAP = {}

CHROME_APP_OPEN_CMD_MacOS = 'open -a /Applications/Google\ Chrome.app/Contents/MacOS/Google\ Chrome {filepath}'
CHROME_APP_OPEN_CMD_LINUX = None    # TODO Linux Chrome 打开文件cmd
CHROME_APP_OPEN_CMD_WINDOWS = None  # TODO Windows Chrome 打开文件cmd
CHROME_APP_OPEN_CMD = CHROME_APP_OPEN_CMD_MacOS

LOGGING = {
    'version': 1,
    'formatters': {
        'default': {
            'format': '%(asctime)s|%(levelname)s|%(message)s'
        },
        'app': {
            'format': '%(asctime)s|%(levelname)s|%(module)s|%(funcName)s|%(lineno)d|%(message)s'
        },
    },
    'handlers': {
        'console': {
            'class': 'logging.StreamHandler',
            'formatter': 'default',
            'level': 'INFO',
        },
        'app': {
            'class': 'logging.handlers.WatchedFileHandler',
            'filename': 'app.log',
            'formatter': 'app',
            'level': 'DEBUG',
        }
    },
    'loggers': {
        'booking': {
            'handlers': ['console', 'app'],
            'level': os.getenv('BOOKING_LOG_LEVEL', 'INFO'),
        }
    },
}
